@@ -1,13 +1,13 @@
 #############################################################################
 #
-#	$Id: naming.py,v 2.64.2.24 2009/04/28 18:18:27 irmen Exp $
 #	Pyro Name Server
 #
 #	This is part of "Pyro" - Python Remote Objects
-#	which is (c) Irmen de Jong - irmen@users.sourceforge.net
+#	which is (c) Irmen de Jong - irmen@razorvine.net
 #
 #############################################################################
 
+from __future__ import with_statement
 import sys, os, socket, time, traceback, errno
 import dircache, shutil, SocketServer
 import Pyro.constants, Pyro.core, Pyro.errors, Pyro.protocol, Pyro.util
@@ -29,7 +29,7 @@ Log = Pyro.util.Log
 #
 #############################################################################
 
-class NameServerLocator:
+class NameServerLocator(object):
 	def __init__(self, identification=None):
 		Pyro.core._checkInit()	# init required
 		self.identification=identification
@@ -190,14 +190,14 @@ class NameServerLocator:
 # names to absolute names. Because this is done in the proxy, each
 # client can have its own PYRO_NS_DEFAULTGROUP.
 
-class NameServerProxy:
+class NameServerProxy(object):
 	def __init__(self,URI,identification=None,noconnect=0):
 		self.URI = URI
 		self.objectID = URI.objectID
 		self.adapter = Pyro.protocol.getProtocolAdapter(self.URI.protocol)
 		self.adapter.setIdentification(identification)
 		if noconnect:
-			self.adapter.URI=URI.clone()
+			self.adapter.URI=URI
 		else:
 			self.adapter.bindToURI(URI)
 		self.adapter.setOneway(["_synccall"])
@@ -350,8 +350,7 @@ class NameServer(Pyro.core.ObjBase):
 	def register(self,name,URI):
 		(origname,name)=name,self.validateName(name)
 		URI=self.validateURI(URI)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			(group, name)=self.locateGrpAndName(name)
 			if len(name or "")<1:
 				raise Pyro.errors.NamingError('invalid name',origname)
@@ -364,13 +363,10 @@ class NameServer(Pyro.core.ObjBase):
 			except KeyError:
 				Log.msg('NameServer','name already exists:',name)
 				raise Pyro.errors.NamingError('name already exists',name)
-		finally:
-			self.lock.release()
 
 	def unregister(self,name):
 		(origname,name)=name,self.validateName(name)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			(group, name)=self.locateGrpAndName(name)
 			if len(name or "")<1:
 				raise Pyro.errors.NamingError('invalid name',origname)
@@ -383,8 +379,6 @@ class NameServer(Pyro.core.ObjBase):
 			except ValueError:
 				Log.msg('NameServer','attempt to remove a group:',name)
 				raise Pyro.errors.NamingError('is a group, not an object',name)
-		finally:
-			self.lock.release()
 
 	def resolve(self,name):
 		# not thread-locked: higher performance and not necessary.
@@ -403,11 +397,8 @@ class NameServer(Pyro.core.ObjBase):
 
 	def flatlist(self):
 		# return a dump
-		self.lock.acquire()
-		try:
+		with self.lock:
 			r=self.root.flatten()
-		finally:
-			self.lock.release()
 		for i in xrange(len(r)):
 			r[i]=(':'+r[i][0], r[i][1])
 		return r
@@ -424,11 +415,8 @@ class NameServer(Pyro.core.ObjBase):
 			Log.msg("NameServer","resync requested from NS at",twinProxy.URI.address,"port",twinProxy.URI.port)
 			print "Resync requested from NS at",twinProxy.URI.address,"port",twinProxy.URI.port
 			self.otherNS=twinProxy
-			self.lock.acquire()
-			try:
+			with self.lock:
 				return self._getSyncDump()
-			finally:
-				self.lock.release()
 		else:
 			Log.warn("NameServer","resync requested from",twinProxy.URI,"but not running in correct mode")
 			raise Pyro.errors.NamingError("The (other) NS is not running in 'primary' or 'secondary' mode")
@@ -465,8 +453,7 @@ class NameServer(Pyro.core.ObjBase):
 		groupname=self.validateName(groupname)
 		if len(groupname)<2:
 			raise Pyro.errors.NamingError('invalid groupname', groupname)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			(parent,name)=self.locateGrpAndName(groupname)
 			if isinstance(parent,NameValue):
 				raise Pyro.errors.NamingError('parent is no group', groupname)
@@ -476,16 +463,13 @@ class NameServer(Pyro.core.ObjBase):
 				self._dosynccall("createGroup",groupname)
 			except KeyError:
 				raise Pyro.errors.NamingError('group already exists',name)
-		finally:
-			self.lock.release()
 
 	def deleteGroup(self,groupname):
 		groupname=self.validateName(groupname)
 		if groupname==':':
 			Log.msg('NameServer','attempt to deleteGroup root group')
 			raise Pyro.errors.NamingError('not allowed to delete root group')
-		self.lock.acquire()
-		try:
+		with self.lock:
 			(parent,name)=self.locateGrpAndName(groupname)
 			try:
 				parent.cutbranch(name)
@@ -495,8 +479,6 @@ class NameServer(Pyro.core.ObjBase):
 				raise Pyro.errors.NamingError('group not found',groupname)
 			except ValueError:
 				raise Pyro.errors.NamingError('is no group',groupname)
-		finally:
-			self.lock.release()
 			
 	def list(self,groupname):
 		# not thread-locked: higher performance and not necessary.
@@ -629,7 +611,7 @@ class NameServer(Pyro.core.ObjBase):
 #
 #############################################################################
 
-class NameSpaceSystemMeta:
+class NameSpaceSystemMeta(object):
 	def __init__(self, node, timestamp, owner):
 		self.timestamp=timestamp
 		self.owner=owner
@@ -643,7 +625,7 @@ class NameSpaceSystemMeta:
 		
 # All nodes in the namespace (groups, or namevalue pairs--leafs) have
 # a shared set of properties, most notably: meta information.
-class NameSpaceNode:
+class NameSpaceNode(object):
 	def __init__(self, name, meta, owner):
 		self.name=name
 		self.systemMeta = NameSpaceSystemMeta(self, time.time(), owner)
@@ -801,8 +783,7 @@ class PersistentNameServer(NameServer):
 		origname,name=name,self.validateName(name)
 		URI=self.validateURI(URI)
 		fn=self.translate(name)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			if os.access(fn,os.R_OK):
 				Log.msg('NameServer','name already exists:',name)
 				raise Pyro.errors.NamingError('name already exists',name)
@@ -817,14 +798,11 @@ class PersistentNameServer(NameServer):
 					raise Pyro.errors.NamingError('parent is no group')
 				else:
 					raise Pyro.errors.NamingError(str(x))
-		finally:
-			self.lock.release()
 
 	def unregister(self,name):
 		origname,name=name,self.validateName(name)
 		fn=self.translate(name)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			try:
 				os.remove(fn)
 				self._dosynccall("unregister",origname)
@@ -837,8 +815,6 @@ class PersistentNameServer(NameServer):
 					raise Pyro.errors.NamingError('is a group, not an object',name)
 				else:
 					raise Pyro.errors.NamingError(str(x))
-		finally:
-			self.lock.release()
 			
 	def resolve(self,name):
 		# not thread-locked: higher performance and not necessary.
@@ -857,22 +833,18 @@ class PersistentNameServer(NameServer):
 
 	def flatlist(self):
 		dbroot=self.translate(':')
-		self.lock.acquire()
-		try:
+		with self.lock:
 			flat=[]
 			for f in self._filelist(dbroot,dbroot):
 				f=self._unescapefilename(f)
 				flat.append((f, self.resolve(f)))
 			return flat
-		finally:
-			self.lock.release()
 
 	# --- hierarchical naming support
 	def createGroup(self,groupname):
 		groupname=self.validateName(groupname)
 		dirnam = self.translate(groupname)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			try:
 				os.mkdir(dirnam)
 				self._dosynccall("createGroup",groupname)
@@ -884,8 +856,6 @@ class PersistentNameServer(NameServer):
 					raise Pyro.errors.NamingError('(parent)group not found')
 				else:
 					raise Pyro.errors.NamingError(str(x))
-		finally:
-			self.lock.release()
 
 	def deleteGroup(self,groupname):
 		groupname=self.validateName(groupname)
@@ -893,8 +863,7 @@ class PersistentNameServer(NameServer):
 			Log.msg('NameServer','attempt to deleteGroup root group')
 			raise Pyro.errors.NamingError('not allowed to delete root group')
 		dirnam = self.translate(groupname)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			if not os.access(dirnam,os.R_OK):
 				raise Pyro.errors.NamingError('group not found',groupname)
 			try:
@@ -908,16 +877,13 @@ class PersistentNameServer(NameServer):
 					raise Pyro.errors.NamingError('is no group',groupname)
 				else:
 					raise Pyro.errors.NamingError(str(x))
-		finally:
-			self.lock.release()
 			
 	def list(self,groupname):
 		if not groupname:
 			groupname=':'
 		groupname=self.validateName(groupname)
 		dirnam=self.translate(groupname)
-		self.lock.acquire()
-		try:
+		with self.lock:
 			if os.access(dirnam,os.R_OK):
 				if os.path.isfile(dirnam):
 					raise Pyro.errors.NamingError('is no group',groupname)
@@ -935,8 +901,6 @@ class PersistentNameServer(NameServer):
 								entries.append((objname,1))		# leaf has code 1
 					return entries
 			raise Pyro.errors.NamingError('group not found',groupname)
-		finally:
-			self.lock.release()
 
 
 	# --- private methods follow
@@ -1027,7 +991,6 @@ class PersistentNameSpaceNode(NameSpaceNode):
 		NameSpaceNode.__init__(self, name, meta, owner)
 		self.filename=filename
 		if not name:
-			# print "INIT PERSISTENT NODE FROM FILE",self.filename
 			# init from file
 			try:
 				(sysmeta, usermeta)=Pyro.util.getPickle().load(open(self.filename,"rb"))
@@ -1036,7 +999,6 @@ class PersistentNameSpaceNode(NameSpaceNode):
 			except Exception:
 				pass # just use empty meta...
 		else:
-			# write to file
 			self._writeToFile()
 	def setMeta(self,meta):
 		NameSpaceNode.setMeta(self, meta)
@@ -1045,7 +1007,6 @@ class PersistentNameSpaceNode(NameSpaceNode):
 		NameSpaceNode.setSystemMeta(self, meta)
 		self._writeToFile()
 	def _writeToFile(self):
-		# print "WRITE PERSISTENT NODE TO FILE",self.filename
 		Pyro.util.getPickle().dump( (self.getSystemMeta(), self.getMeta()) , open(self.filename,"wb"), Pyro.config.PYRO_PICKLE_FORMAT)
 		
 
@@ -1122,7 +1083,7 @@ class bcRequestHandler(SocketServer.BaseRequestHandler):
 
 # The default BC request validator... accepts everything
 # You must subclass this for your own validators
-class BCReqValidator:
+class BCReqValidator(object):
 	def __call__(self, req, addr):
 		(cmd,self.sock)=req
 		self.addr=addr
@@ -1142,9 +1103,10 @@ class BCReqValidator:
 
 #############################################################################
 
-class NameServerStarter:
+class NameServerStarter(object):
 	def __init__(self, identification=None):
 		Pyro.core.initServer()
+		self.persistent=False
 		self.identification=identification
 		self.started = Pyro.util.getEventObject()
 	def start(self, *args, **kwargs):			# see _start for allowed arguments
@@ -1233,10 +1195,12 @@ class NameServerStarter:
 			ns=PersistentNameServer(dbdir,role=role[0], identification=self.identification)
 			daemon.useNameServer(ns)
 			NS_URI=daemon.connectPersistent(ns,Pyro.constants.NAMESERVER_NAME)
+			self.persistent=True
 		else:
 			ns=NameServer(role=role[0], identification=self.identification)
 			daemon.useNameServer(ns)
 			NS_URI=daemon.connect(ns,Pyro.constants.NAMESERVER_NAME)
+			self.persistent=False
 
 		self.bcserver=None
 		if nobroadcast:
@@ -1433,7 +1397,8 @@ class NameServerStarter:
 			ns=daemon.getNameServer()
 			del self.daemon
 		ns._removeTwinNS()
-		daemon.disconnect(ns) # clean up nicely
+		if not self.persistent:
+			daemon.disconnect(ns) # clean up nicely only if not running in persistent mode
 		if self.bcserver:
 			self.bcserver.shutdown=1
 		daemon.shutdown()
@@ -1443,7 +1408,7 @@ def main(argv):
 	Args.parse(argv,'hkmrvxn:p:b:c:d:s:i:1:2:')
 	if Args.hasOpt('h'):
 		print 'Usage: pyro-ns [-h] [-k] [-m] [-r] [-x] [-n hostname] [-p port] [-b bcport] [-c bcaddr]'
-		print '          [-i identification] [-d [databasefile]] [-s securitymodule]'
+		print '          [-i identification] [-d [databaselocation]] [-s securitymodule]'
 		print '          [-1 [host:port]] [-2 [host:port]] [-v]'
 		print '  where -p = NS server port (0 for auto)'
 		print '        -n = non-default hostname to bind on'
