@@ -1,5 +1,5 @@
 
-class cd:
+class cd(object):
 	def __init__(self):
 		pass
 	def getName(self):
